#ifndef __ASM_CPUCAPS_H
#define __ASM_CPUCAPS_H

/* Generated file - do not edit */

#define ARM64_ALWAYS_BOOT                   	0
#define ARM64_ALWAYS_SYSTEM                 	1
#define ARM64_BTI                           	2
#define ARM64_HAS_32BIT_EL0_DO_NOT_USE      	3
#define ARM64_HAS_32BIT_EL1                 	4
#define ARM64_HAS_ADDRESS_AUTH              	5
#define ARM64_HAS_ADDRESS_AUTH_ARCH_QARMA3  	6
#define ARM64_HAS_ADDRESS_AUTH_ARCH_QARMA5  	7
#define ARM64_HAS_ADDRESS_AUTH_IMP_DEF      	8
#define ARM64_HAS_AMU_EXTN                  	9
#define ARM64_HAS_ARMv8_4_TTL               	10
#define ARM64_HAS_CACHE_DIC                 	11
#define ARM64_HAS_CACHE_IDC                 	12
#define ARM64_HAS_CNP                       	13
#define ARM64_HAS_CRC32                     	14
#define ARM64_HAS_DCPODP                    	15
#define ARM64_HAS_DCPOP                     	16
#define ARM64_HAS_DIT                       	17
#define ARM64_HAS_E0PD                      	18
#define ARM64_HAS_ECV                       	19
#define ARM64_HAS_EPAN                      	20
#define ARM64_HAS_GENERIC_AUTH              	21
#define ARM64_HAS_GENERIC_AUTH_ARCH_QARMA3  	22
#define ARM64_HAS_GENERIC_AUTH_ARCH_QARMA5  	23
#define ARM64_HAS_GENERIC_AUTH_IMP_DEF      	24
#define ARM64_HAS_GIC_CPUIF_SYSREGS         	25
#define ARM64_HAS_GIC_PRIO_MASKING          	26
#define ARM64_HAS_GIC_PRIO_RELAXED_SYNC     	27
#define ARM64_HAS_LDAPR                     	28
#define ARM64_HAS_LSE_ATOMICS               	29
#define ARM64_HAS_NESTED_VIRT               	30
#define ARM64_HAS_NO_FPSIMD                 	31
#define ARM64_HAS_NO_HW_PREFETCH            	32
#define ARM64_HAS_PAN                       	33
#define ARM64_HAS_RAS_EXTN                  	34
#define ARM64_HAS_RNG                       	35
#define ARM64_HAS_SB                        	36
#define ARM64_HAS_STAGE2_FWB                	37
#define ARM64_HAS_TIDCP1                    	38
#define ARM64_HAS_TLB_RANGE                 	39
#define ARM64_HAS_TSO_APPLE                 	40
#define ARM64_HAS_TSO_FIXED                 	41
#define ARM64_HAS_VIRT_HOST_EXTN            	42
#define ARM64_HAS_WFXT                      	43
#define ARM64_HW_DBM                        	44
#define ARM64_KVM_PROTECTED_MODE            	45
#define ARM64_MISMATCHED_CACHE_TYPE         	46
#define ARM64_MTE                           	47
#define ARM64_MTE_ASYMM                     	48
#define ARM64_SME                           	49
#define ARM64_SME_FA64                      	50
#define ARM64_SME2                          	51
#define ARM64_SPECTRE_V2                    	52
#define ARM64_SPECTRE_V3A                   	53
#define ARM64_SPECTRE_V4                    	54
#define ARM64_SPECTRE_BHB                   	55
#define ARM64_SSBS                          	56
#define ARM64_SVE                           	57
#define ARM64_UNMAP_KERNEL_AT_EL0           	58
#define ARM64_WORKAROUND_834220             	59
#define ARM64_WORKAROUND_843419             	60
#define ARM64_WORKAROUND_845719             	61
#define ARM64_WORKAROUND_858921             	62
#define ARM64_WORKAROUND_1418040            	63
#define ARM64_WORKAROUND_1463225            	64
#define ARM64_WORKAROUND_1508412            	65
#define ARM64_WORKAROUND_1542419            	66
#define ARM64_WORKAROUND_1742098            	67
#define ARM64_WORKAROUND_1902691            	68
#define ARM64_WORKAROUND_2038923            	69
#define ARM64_WORKAROUND_2064142            	70
#define ARM64_WORKAROUND_2077057            	71
#define ARM64_WORKAROUND_2457168            	72
#define ARM64_WORKAROUND_2645198            	73
#define ARM64_WORKAROUND_2658417            	74
#define ARM64_WORKAROUND_TRBE_OVERWRITE_FILL_MODE	75
#define ARM64_WORKAROUND_TSB_FLUSH_FAILURE  	76
#define ARM64_WORKAROUND_TRBE_WRITE_OUT_OF_RANGE	77
#define ARM64_WORKAROUND_CAVIUM_23154       	78
#define ARM64_WORKAROUND_CAVIUM_27456       	79
#define ARM64_WORKAROUND_CAVIUM_30115       	80
#define ARM64_WORKAROUND_CAVIUM_TX2_219_PRFM	81
#define ARM64_WORKAROUND_CAVIUM_TX2_219_TVM 	82
#define ARM64_WORKAROUND_CLEAN_CACHE        	83
#define ARM64_WORKAROUND_DEVICE_LOAD_ACQUIRE	84
#define ARM64_WORKAROUND_NVIDIA_CARMEL_CNP  	85
#define ARM64_WORKAROUND_QCOM_FALKOR_E1003  	86
#define ARM64_WORKAROUND_REPEAT_TLBI        	87
#define ARM64_WORKAROUND_SPECULATIVE_AT     	88
#define ARM64_NCAPS				89

#endif /* __ASM_CPUCAPS_H */
