/* SPDX-License-Identifier: GPL-2.0-only OR BSD-2-Clause */
/*
 * Copyright (c) 2010-2015, 2018-2019 The Linux Foundation. All rights reserved.
 * Copyright (C) 2015 Linaro Ltd.
 */

#ifndef _DT_BINDINGS_FIRMWARE_QCOM_SCM_H
#define _DT_BINDINGS_FIRMWARE_QCOM_SCM_H

#define QCOM_SCM_VMID_HLOS		0x3
#define QCOM_SCM_VMID_MSS_MSA		0xF
#define QCOM_SCM_VMID_WLAN		0x18
#define QCOM_SCM_VMID_WLAN_CE		0x19
#define QCOM_SCM_VMID_NAV		0x2B

#endif
