/* SPDX-License-Identifier: (GPL-2.0+ or MIT) */
/*
 * Copyright (C) 2021 Samuel Holland <samuel@sholland.org>
 */

#ifndef _DT_BINDINGS_CLK_SUN20I_D1_R_CCU_H_
#define _DT_BINDINGS_CLK_SUN20I_D1_R_CCU_H_

#define CLK_R_AHB		0

#define CLK_BUS_R_TIMER		2
#define CLK_BUS_R_TWD		3
#define CLK_BUS_R_PPU		4
#define CLK_R_IR_RX		5
#define CLK_BUS_R_IR_RX		6
#define CLK_BUS_R_RTC		7
#define CLK_BUS_R_CPUCFG	8

#endif /* _DT_BINDINGS_CLK_SUN20I_D1_R_CCU_H_ */
